import click

option = click.option(
    '--alias',
    '-a',
    'configure_alias',
    default=None,
    help='Optional alias for the above tenant. This alias would be used with the `--tenant` flag.',
)



import click

# eval example: eval $(pybritive checkout test -m env)

# explanations:
#     awscredentialprocess: aws credential process output with additional caching to make the credential process more
#         performant
#     azlogin: azure az login command with all fields populated (suitable for eval)
#     azps: azure powershell script
#     browser: checkout console access (without having to specify --console/-c) and open the default browser to the URL
#         provided in the checkout
#     console: checkout console access (without having to specify --console/-c) and print the URL
#     env: environment variable output (suitable for eval)
#     env-nix: environment variable output specifying "export"
#     env-wincmd: environment variable output specifying "set"
#     env-winps: environment variable output specifying "$env:"
#     gcloudauth: gcloud auth activate-service-account with all fields populated (suitable for eval)
#     gcloudauthexec: will effectively execute results of gcloudauth in a sub-shell
#     integrate: stick the credentials into the local CSP credentials file (AWS)
#     json: json based output
#     kube-exec: bake into kubeconfig with oidc exec output and additional caching to make kubectl more performant
#     os-oclogin: will attempt an oidc authorization code grant flow for generating the `oc login ...` command for
#         OpenShift
#     os-ocloginexec: will attempt an oidc authorization code grant flow for generating the `oc login ...` command for
#         OpenShift and exec the result in a subshell
#     text: text based output

mode_choices = click.Choice(
    [
        'awscredentialprocess',
        'azlogin',
        'azps',
        'browser',
        'browser-chrome',
        'browser-chromium',
        'browser-firefox',
        'browser-macosx',
        'browser-mozilla',
        'browser-safari',
        'browser-windows-default',
        'console',
        'env',
        'env-nix',
        'env-wincmd',
        'env-winps',
        'gcloudauth',
        'gcloudauthexec',
        'integrate',
        'json',
        'kube-exec',
        'os-oclogin',
        'os-ocloginexec',
        'text',
    ],
    case_sensitive=False,
)



import click

option = click.option(
    '--extend',
    '-e',
    default=False,
    is_flag=True,
    show_default=True,
    help='Extend the expiration time for a currently checked out profile.',
)



import click

option = click.option(
    '--profile',
    '-p',
    default=None,
    help='AWS CLI profile from which credentials will be sourced. Defaults to None which indicates that the default '
    'credential provider chain should be used.',
    show_envvar=True,
    show_default=True,
)



import click

option = click.option(
    '--justification',
    '-j',
    default=None,
    show_default=True,
    help='Justification for the approval process, if a profile checkout or secret access requires approval.',
)



import click

option = click.option(
    '--gcloud-key-file',
    default=None,
    help=(
        'GCP Programmatic Only - When mode is `gcloudauth` specify a non-default location for storing the key file.'
        ' Ignored for mode `gcloudauthexec`.'
    ),
    show_default=True,
)



import click

option = click.option(
    '--file',
    '-F',
    help='Path to a file where the contents of the secret file will be stored. Omitting this option will '
    'result in the file being saved to the current directory with the name provided when the secret '
    'file was initially uploaded. Providing `-` will print the contents of the secret file to stdout.',
)



import click

option = click.option(
    '--otp',
    '-o',
    default=None,
    show_default=False,
    help='OTP to checkout a profile, download a secret, or view a secret, if MFA is required.',
)



import os

PROFILE = os.getenv('PYBRITIVE_TEST_PROFILE')


def test_checkin(runner, cli):
    result = runner.invoke(cli, ['checkin', PROFILE])
    assert result.exit_code == 0
    assert result.output == ''



import contextlib
import io
import os
import sys
from sys import argv


def _fallback_input(prompt='', stream=None):
    if not stream:
        stream = sys.stderr
    return _raw_input(prompt, stream)


def _linput(prompt='', stream=None):
    user_input = None
    with contextlib.ExitStack() as stack:
        try:
            fd = os.open('/dev/tty', os.O_RDWR | os.O_NOCTTY)
            tty = io.FileIO(fd, 'w+')
            stack.enter_context(tty)
            std_input = io.TextIOWrapper(tty)
            stack.enter_context(std_input)
            if not stream:
                stream = std_input
        except OSError:
            stack.close()
            try:
                fd = sys.stdin.fileno()
            except (AttributeError, ValueError):
                fd = None
                user_input = _fallback_input(prompt, stream)
            std_input = sys.stdin
            if not stream:
                stream = sys.stderr

        if fd is not None:
            try:
                old = termios.tcgetattr(fd)
                new = old[:]
                new[3] |= termios.ECHO
                tcsetattr_flags = termios.TCSAFLUSH
                if hasattr(termios, 'TCSASOFT'):
                    tcsetattr_flags |= termios.TCSASOFT
                try:
                    termios.tcsetattr(fd, tcsetattr_flags, new)
                    user_input = _raw_input(prompt, stream, std_input=std_input)
                finally:
                    termios.tcsetattr(fd, tcsetattr_flags, old)
                    stream.flush()
            except termios.error:
                if user_input is not None:
                    raise
                if stream is not std_input:
                    stack.close()
                user_input = _fallback_input(prompt, stream)

        stream.write('\n')
        return user_input


def _raw_input(prompt='', stream=None, std_input=None):
    if not stream:
        stream = sys.stderr
    if not std_input:
        std_input = sys.stdin
    prompt = str(prompt)
    if prompt:
        try:
            stream.write(prompt)
        except UnicodeEncodeError:
            prompt = prompt.encode(stream.encoding, 'replace')
            prompt = prompt.decode(stream.encoding)
            stream.write(prompt)
        stream.flush()
    line = std_input.readline()
    if not line:
        raise EOFError
    if line[-1] == '\n':
        line = line[:-1]
    return line


def _winput(prompt='', stream=None):
    if sys.stdin is not sys.__stdin__:
        return _fallback_input(prompt, stream)

    for c in prompt:
        msvcrt.putwch(c)
    user_input = ''
    while 1:
        c = msvcrt.getwche()
        if c in ('\r', '\n'):
            break
        if c == '\003':
            raise KeyboardInterrupt
        user_input = user_input[:-1] if c == '\x08' else user_input + c
    msvcrt.putwch('\r')
    msvcrt.putwch('\n')
    return user_input


try:
    import termios

    termios.tcgetattr, termios.tcsetattr  # noqa: B018
except (ImportError, AttributeError):
    try:
        import msvcrt
    except ImportError:
        get_input = _fallback_input
    else:
        get_input = _winput
else:
    get_input = _linput


def get_args():
    from getopt import getopt  # lazy load

    options = getopt(
        argv[1:],
        't:T:p:f:P:F:hv',
        ['tenant=', 'token=', 'passphrase=', 'force-renew=', 'profile=', 'federation-provider=', 'help', 'version'],
    )[0]

    args = {
        'tenant': None,
        'token': None,
        'passphrase': None,
        'force_renew': None,
        'profile': None,
        'federation_provider': None,
    }

    for opt, arg in options:
        if opt in ('-t', '--tenant'):
            args['tenant'] = arg
        if opt in ('-T', '--token'):
            args['token'] = arg
        if opt in ('-p', '--passphrase'):
            args['passphrase'] = arg
        if opt in ('-f', '--force-renew'):
            args['force_renew'] = int(arg)
        if opt in ('-P', '--profile'):
            args['profile'] = arg
        if opt in ('-F', '--federation-provider'):
            args['federation_provider'] = arg
        if opt in ('-h', '--help'):
            usage()
        if opt in ('-v', '--version'):
            from importlib.metadata import version
            from platform import platform, python_version  # lazy load

            cli_version = version('pybritive')
            print(f'pybritive: {cli_version} / platform: {platform()} / python: {python_version()}')
            raise SystemExit

    return args


def usage():
    print(
        f'Usage : {argv[0]} -P/--profile <profile> [-t/--tenant, -T/--token, -p/--passphrase, -f/--force-renew,'
        ' -F/--federation-provider]'
    )
    raise SystemExit


def perform_checkout(b, args, otp=None, justification=None):
    b.checkout(
        alias=None,
        blocktime=None,
        console=False,
        justification=justification,
        mode='awscredentialprocess',
        maxpolltime=None,
        profile=args['profile'],
        passphrase=args['passphrase'],
        force_renew=args['force_renew'],
        aws_credentials_file=None,
        gcloud_key_file=None,
        verbose=None,
        extend=False,
        otp=otp,
    )


def main():
    args = get_args()
    if not args['profile']:
        print('-P/--profile is required')
        usage()

    creds = None
    if not args['force_renew']:  # if force renew let's defer to that the full package vs. this helper
        from pybritive.helpers.cache import Cache  # lazy load

        creds = Cache(passphrase=args['passphrase']).get_credentials(
            profile_name=args['profile'], mode='awscredentialprocess'
        )
        if creds:
            from datetime import datetime  # lazy load

            expiration = datetime.fromisoformat(creds['expirationTime'].replace('Z', ''))
            now = datetime.utcnow()
            if now > expiration:  # creds have expired so set to none so new one get checked out
                creds = None
            else:  # not importing json library on purpose to keep imports down for speed
                json = '{'
                json += f'"AccessKeyId": "{creds["accessKeyID"]}",'
                json += f'"SecretAccessKey": "{creds["secretAccessKey"]}",'
                json += f'"SessionToken": "{creds["sessionToken"]}",'
                json += f'"Expiration": "{creds["expirationTime"]}",'
                json += '"Version": 1}'
                print(json)
                raise SystemExit
    if not creds:
        from britive import exceptions

        from pybritive.britive_cli import BritiveCli  # lazy load for performance purposes

        b = BritiveCli(
            tenant_name=args['tenant'],
            token=args['token'],
            passphrase=args['passphrase'],
            federation_provider=args['federation_provider'],
            silent=True,
            from_helper_console_script=True,
        )
        b.config.get_tenant()  # have to load the config here as that work is generally done

        try:
            perform_checkout(b, args)
        except exceptions.StepUpAuthRequiredButNotProvided:
            perform_checkout(b, args, otp=get_input(prompt='(pybritive) Enter OTP:'))
        except (
            exceptions.ApprovalRequiredButNoJustificationProvided,
            exceptions.badrequest.MissingJustificationError,
        ):
            try:
                perform_checkout(b, args, justification=get_input(prompt='(pybritive) Enter Justification: '))
            except exceptions.ProfileApprovalMaxBlockTimeExceeded as e:
                b.request_withdraw(profile=args['profile'])
                raise SystemExit('approval not settled before blocktime exceeded - request withdrawn') from e

        raise SystemExit


if __name__ == '__main__':
    main()



import click

from pybritive.helpers.build_britive import build_britive
from pybritive.helpers.profile_argument_decorator import click_smart_profile_argument
from pybritive.options.britive_options import britive_options


@click.command()
@build_britive
@britive_options(
    names='alias,blocktime,console,justification,ticket_type,ticket_id,otp,mode,maxpolltime,silent,force_renew,aws_credentials_file,'
    'gcloud_key_file,verbose,extend,profile_type,tenant,token,passphrase,federation_provider'
)
@click_smart_profile_argument
def checkout(  # noqa: PLR0913
    ctx,
    alias,
    blocktime,
    console,
    justification,
    ticket_type,
    ticket_id,
    otp,
    mode,
    maxpolltime,
    silent,
    force_renew,
    aws_credentials_file,
    gcloud_key_file,
    verbose,
    extend,
    profile_type,
    tenant,
    token,
    passphrase,
    federation_provider,
    profile,
):
    """Checkout a profile.

    This command takes 1 required argument `PROFILE`. This should be a string representation of the profile
    that should be checked out. Format is `application name/environment name/profile name`.
    """

    # silent will get passed in via @build_britive
    ctx.obj.britive.checkout(
        alias=alias,
        blocktime=blocktime,
        console=console,
        justification=justification,
        ticket_type=ticket_type,
        ticket_id=ticket_id,
        otp=otp,
        mode=mode,
        maxpolltime=maxpolltime,
        profile=profile,
        passphrase=passphrase,
        force_renew=force_renew,
        aws_credentials_file=aws_credentials_file,
        gcloud_key_file=gcloud_key_file,
        verbose=verbose,
        extend=extend,
        profile_type=profile_type,
    )



import click

from pybritive.options.alias import option as alias
from pybritive.options.aws_console_duration import option as aws_console_duration
from pybritive.options.aws_credentials_file import option as aws_credentials_file
from pybritive.options.aws_profile import option as aws_profile
from pybritive.options.blocktime import option as blocktime
from pybritive.options.browser import option as browser
from pybritive.options.checked_out import option as checked_out
from pybritive.options.configure_alias import option as configure_alias
from pybritive.options.configure_backend import option as configure_backend
from pybritive.options.configure_prompt import option as configure_prompt
from pybritive.options.configure_tenant import option as configure_tenant
from pybritive.options.console import option as console
from pybritive.options.extend import option as extend
from pybritive.options.federation_provider import option as federation_provider
from pybritive.options.file import option as file
from pybritive.options.force_renew import option as force_renew
from pybritive.options.gcloud_key_file import option as gcloud_key_file
from pybritive.options.justification import option as justification
from pybritive.options.maxpolltime import option as maxpolltime
from pybritive.options.mode import option as mode
from pybritive.options.otp import option as otp
from pybritive.options.output_format import option as output_format
from pybritive.options.passphrase import option as passphrase
from pybritive.options.profile_type import option as profile_type
from pybritive.options.query import option as query
from pybritive.options.silent import option as silent
from pybritive.options.ssh_hostname import option as ssh_hostname
from pybritive.options.ssh_key_source import option as ssh_key_source
from pybritive.options.ssh_port import option as ssh_port
from pybritive.options.ssh_push_public_key import option as ssh_push_public_key
from pybritive.options.ssh_username import option as ssh_username
from pybritive.options.tenant import option as tenant
from pybritive.options.ticket_id import option as ticket_id
from pybritive.options.ticket_type import option as ticket_type
from pybritive.options.token import option as token
from pybritive.options.verbose import option as verbose
from pybritive.options.version import option as version

options_map = {
    'tenant': tenant,
    'token': token,
    'format': output_format,
    'output_format': output_format,
    'alias': alias,
    'blocktime': blocktime,
    'mode': mode,
    'maxpolltime': maxpolltime,
    'version': version,
    'configure_tenant': configure_tenant,
    'configure_alias': configure_alias,
    'configure_prompt': configure_prompt,
    'justification': justification,
    'ticket_type': ticket_type,
    'ticket_id': ticket_id,
    'otp': otp,
    'silent': silent,
    'console': console,
    'checked_out': checked_out,
    'file': file,
    'configure_backend': configure_backend,
    'passphrase': passphrase,
    'force_renew': force_renew,
    'aws_credentials_file': aws_credentials_file,
    'federation_provider': federation_provider,
    'gcloud_key_file': gcloud_key_file,
    'verbose': verbose,
    'query': query,
    'ssh_hostname': ssh_hostname,
    'ssh_username': ssh_username,
    'ssh_push_public_key': ssh_push_public_key,
    'ssh_port': ssh_port,
    'ssh_key_source': ssh_key_source,
    'aws_profile': aws_profile,
    'aws_console_duration': aws_console_duration,
    'browser': browser,
    'extend': extend,
    'profile_type': profile_type,
}


def britive_options(**kwargs):
    def inner(f):
        names = [n.strip() for n in kwargs['names'].split(',')]
        names.reverse()
        for name in names:
            option = options_map.get(name)
            if not name:
                raise click.ClickException(f'Invalid option {name} provided.')
            f = option(f)
        return f

    return inner



import json
import os

PROFILE = os.getenv('PYBRITIVE_TEST_PROFILE')


def test_checkout_json(runner, cli):
    result = runner.invoke(cli, ['checkout', PROFILE, '-m', 'json'])
    assert result.exit_code == 0
    creds = json.loads(result.output)
    assert 'AccessKeyId' in creds
    assert 'SecretAccessKey' in creds
    assert 'SessionToken' in creds
    assert 'Expiration' in creds


def test_checkout_aws_credential_process(runner, cli):
    result = runner.invoke(cli, ['checkout', PROFILE, '-m', 'awscredentialprocess'])
    assert result.exit_code == 0
    creds = json.loads(result.output)
    assert 'AccessKeyId' in creds
    assert 'SecretAccessKey' in creds
    assert 'SessionToken' in creds
    assert 'Expiration' in creds
    assert 'Version' in creds



import json
import os

import click

from pybritive.completers.profile import profile_completer


def validate_profile(ctx, param, value):
    if kube_exec_info := os.getenv('KUBERNETES_EXEC_INFO'):
        try:
            return json.loads(kube_exec_info)['spec']['cluster']['config']['britive-profile']
        except Exception as e:
            raise ValueError(
                'unable to find britive profile via cluster exec-extension with name britive-profile'
            ) from e
    return value


def is_required():
    return 'KUBERNETES_EXEC_INFO' not in os.environ


def click_smart_profile_argument(func):
    required = is_required()
    kwargs = {'required': required}
    if not required:
        kwargs['callback'] = validate_profile
    kwargs['shell_complete'] = profile_completer

    dec = click.argument('profile', **kwargs)
    return dec(func)



from dataclasses import dataclass
from functools import wraps

import click
from click import Context
from merge_args import merge_args

from pybritive.britive_cli import BritiveCli


@dataclass
class Common:
    britive: BritiveCli


def should_set_output_format(ctx: Context) -> bool:
    parent_command = ctx.parent.command.name
    command = ctx.command.name
    return not ((parent_command in ['configure', 'clear']) or (parent_command == 'cache' and command in ['clear']))


# this wrapper exists to centralize all "common" CLI options (options that exist for all commands)
def build_britive(f):
    @merge_args(f)
    @wraps(f)
    @click.pass_context
    def wrapper(ctx, **kwargs):
        ctx.obj = Common(
            BritiveCli(
                tenant_name=kwargs.get('tenant'),
                token=kwargs.get('token'),
                silent=kwargs.get('silent', False),
                passphrase=kwargs.get('passphrase'),
                federation_provider=kwargs.get('federation_provider'),
            )
        )
        if should_set_output_format(ctx=ctx):
            ctx.obj.britive.set_output_format(kwargs.get('output_format'))
        return f(ctx=ctx, **kwargs)

    return wrapper
